import cv2
from epic7_bot.templates.BaseTemplates import BaseTemplates
from epic7_bot.templates.Template import Template


class SecretShopTemplates(BaseTemplates):
    def __init__(self):
        super(self.__class__, self).__init__()

        self.buy_button_covenant: Template = {
            "image": cv2.imread(self.dirPath + '/../images/secret_shop/Buy_button_Covenant.png', 0),
            "name": "buy_button_covenant"
        }

        self.buy_button_mystic: Template = {
            "image": cv2.imread(self.dirPath + '/../images/secret_shop/Buy_button_Mystic.png', 0),
            "name": "buy_button_mystic"
        }

        self.covenant: Template = {
            "image": cv2.imread(self.dirPath + '/../images/secret_shop/covenant.png', 0),
            "name": "covenant"
        }

        self.mystic: Template = {
            "image": cv2.imread(self.dirPath + '/../images/secret_shop/mystic.png', 0),
            "name": "mystic"
        }



import cv2
import os
from epic7_bot.templates.BaseTemplates import BaseTemplates
from epic7_bot.templates.Template import Template


class CommonTemplates(BaseTemplates):
    def __init__(self):
        super(self.__class__, self).__init__()

        self.skip_button: Template = {
            "image": cv2.imread(self.dirPath + '/../images/skip_button.png', 0),
            "name": "skip_button"
        }

        self.connecting_problem: Template = {
            "image": cv2.imread(self.dirPath + '/../images/connecting.png', 0),
            "name": "connecting_problem"
        }

        self.there_was_a_connection_error: Template = {
            "image": cv2.imread(self.dirPath + '/../images/there_was_a_connection_error.png', 0),
            "name": "there_was_a_connection_error"
        }

        self.dispatch_mission_completed: Template = {
            "image": cv2.imread(self.dirPath + '/../images/dispatch_mission_completed.png', 0),
            "name": "dispatch_mission_completed"
        }

        self.dispatch_mission: Template = {
            "image": cv2.imread(self.dirPath + '/../images/dispatch_mission.png', 0),
            "name": "dispatch_mission"
        }



from epic7_bot.commands.Command import Command
from epic7_bot.modules.Arena import Arena as ArenaM


class Arena(Command):
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.Arena = ArenaM()

    def start(self):
        if self.currentScreen == False:
            self.Arena.start_arena_npc_auto_battle_from_lobby()
        else:
            self.Arena.start_arena_npc_auto_battle()



import logging
import time
from epic7_bot.modules.Module import Module
from epic7_bot.templates.HuntTemplates import HuntTemplates


class Battle(Module):
    def __init__(self):
        super(self.__class__, self).__init__()
        self.HuntTemplates = HuntTemplates()
        self.total_rotations = 0

    def has_energy(self):
        if self.ScreenManager.match_template_on_screen_area(
                x1=629, y1=175, x2=960, y2=224, template=self.HuntTemplates.insufficient_energy, percentage=0.6) is not None:
            logging.info(f"Insufficient Energy, finishing hunting")
            return False
        return True

    def do_hunt_rotation(self):
        self.show_stats()

        if self.has_energy() == False:
            return

        logging.info(
            f"Wait for stage clear to appear")
        while self.ScreenManager.match_template_on_screen(template=self.HuntTemplates.stage_clear, percentage=0.6) is None: #change this to match other screen
            self.ScreenManager.sleep(1)

        self.total_rotations += 1
       
        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1404, y1=811, x2=1469, y2=842, action="Click on stage clear button")

        if self.ScreenManager.match_template_on_screen_area(
                x1=1395, y1=808, x2=1492, y2=839, template=self.HuntTemplates.confirm, percentage=0.6) is not None:
            self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
                x1=1395, y1=808, x2=1492, y2=839, action="Click on confirm button")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1416, x2=1533, y1=810, y2=841, action="Click on try again button")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1404, y1=811, x2=1469, y2=842, action="Click on start button")



        if self.has_energy() == False:
            return

        self.do_hunt_rotation()

    def show_stats(self):
        comment = f"\nTotal Rotations: {str(self.total_rotations)}{' '*30} \n" \
            + f"{' '*30} \n"
        previus_line = "\033[F"
        if self.total_rotations == 0:
            print(f"\r{previus_line*2}{comment}")
        else:
            print(f"\r{previus_line*4}{comment}")

    def start_hunt(self):
        if self.ScreenManager.match_template_on_screen(template=self.HuntTemplates.pet_auto_battle_active, percentage=0.9) is None:
            self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
                x1=705, y1=675, x2=749, y2=718, action="Click on Pet Auto Battle Active", percentage=20)

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1404, y1=811, x2=1469, y2=842, action="Click on start button")

        self.do_hunt_rotation()

    def start_hunt_from_lobby(self):
        self.ScreenManager.ensure_not_on_sleep_mode_on_lobby()

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1164, y1=803, x2=1254, y2=870, action="Click on Battle button")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=907, y1=648, x2=1055, y2=715, action="Click on Hunt button")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=984, y1=259, x2=1113, y2=336, action="Click on Wyvern Hunt")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1404, y1=810, x2=1559, y2=848, action="Click on Select Team")

        self.start_hunt()



from time import sleep
from epic7_bot.modules.Module import Module


class PetHouse(Module):
    def __init__(self):
        super(self.__class__, self).__init__()

    def get_free_pet_summon(self):
        self.ScreenManager.ensure_not_on_sleep_mode_on_lobby()

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=481, y1=795, x2=582, y2=877, action="Click on Pet House Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1096, y1=795, x2=1197, y2=839, action="Click on First Adopt Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=810, y1=747, x2=923, y2=786, action="Click on Second Adopt Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=699, y1=841, x2=878, y2=879, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=699, y1=841, x2=878, y2=879, action="Click on Tap to Close Again")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=34, y1=12, x2=225, y2=67, action="Go Back to Lobby")



from epic7_bot.commands.Command import Command
from epic7_bot.modules.SecretShop import SecretShop


class Shop(Command):
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.SecretShop = SecretShop()

    def start(self):
        if self.currentScreen == False:
            self.SecretShop.start_auto_buy_secret_shop_from_lobby()
        else:
            self.SecretShop.start_auto_buy_secret_shop()



from time import sleep
from epic7_bot.modules.Module import Module


class Sanctuary(Module):
    def __init__(self):
        super(self.__class__, self).__init__()

    def do_sanctuary_routine(self):
        self.ScreenManager.ensure_not_on_sleep_mode_on_lobby()

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=259, y1=240, x2=359, y2=301, action="Click on Sanctuary Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=708, y1=596, x2=876, y2=661, action="Click on Heart of Orbis Reward")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=722, y1=608, x2=882, y2=647, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=351, y1=365, x2=505, y2=388, action="Click on Forest of Souls")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=663, y1=157, x2=748, y2=223, action="Click on Penguin Nest Reward")

        sleep(2)

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=722, y1=608, x2=882, y2=647, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1251, y1=276, x2=1320, y2=322, action="Click on Spirit Well Reward")

        sleep(2)

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=722, y1=608, x2=882, y2=647, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=915, y1=508, x2=989, y2=573, action="Click on MolaGora Farm Reward")

        sleep(2)

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=722, y1=608, x2=882, y2=647, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=34, y1=12, x2=225, y2=67, action="Go Back to Sanctuary")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=34, y1=12, x2=225, y2=67, action="Go Back to Lobby")



import logging
import multiprocessing
import sys
import time
from epic7_bot.core.Logger import get_log_level
from epic7_bot.core.ScreenManager import ScreenManager
from epic7_bot.templates.CommonTemplates import CommonTemplates
import psutil


class DispatchMissionChecker(multiprocessing.Process):
    def __init__(self, main_process_pid):
        multiprocessing.Process.__init__(self, daemon=True)
        self.CommonTemplates = CommonTemplates()
        self.ScreenManager = ScreenManager()
        self.main_process_pid = main_process_pid

    def match_dispatch_mission_completed_template(self):
        return self.ScreenManager.match_template_on_screen(template=self.CommonTemplates.dispatch_mission_completed, percentage=0.9)

    def match_dispatch_mission_template(self):
        return self.ScreenManager.match_template_on_screen(template=self.CommonTemplates.dispatch_mission, percentage=0.7)

    def run(self):
        try:
            main_process = psutil.Process(self.main_process_pid)
            seconds = 21600  # 6 hours

            # problems with - RecursionError: maximum recursion depth exceeded in comparison
            while seconds != 0:
                seconds -= 1
                time.sleep(1)
                dispatch_mission_completed = self.match_dispatch_mission_completed_template()
                dispatch_mission = self.match_dispatch_mission_template()

                if dispatch_mission is not None or dispatch_mission_completed is not None:
                    main_process.suspend()
                    logging.error(f"Dispatch Mission appeared")
                    time.sleep(5)
                    self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
                        x1=1099, y1=703, x2=1228, y2=745, action="Click on Try Again")
                    time.sleep(2)
                    main_process.resume()
                    break
        except BaseException as e:
            if get_log_level() == "DEBUG":
                raise e
            else:
                pass



import cv2
from epic7_bot.templates.BaseTemplates import BaseTemplates
from epic7_bot.templates.Template import Template


class HuntTemplates(BaseTemplates):
    def __init__(self):
        super(self.__class__, self).__init__()

        self.try_again: Template = {
            "image": cv2.imread(self.dirPath + '/../images/hunt/try_again.png', 0),
            "name": "try_again"
        }

        self.confirm: Template = {
            "image": cv2.imread(self.dirPath + '/../images/hunt/confirm.png', 0),
            "name": "confirm"
        }

        self.insufficient_energy: Template = {
            "image": cv2.imread(self.dirPath + '/../images/hunt/insufficient_energy.png', 0),
            "name": "insufficient_energy"
        }

        self.pet_auto_battle_active: Template = {
            "image": cv2.imread(self.dirPath + '/../images/hunt/pet_auto_battle_active.png', 0),
            "name": "pet_auto_battle_active"
        }

        self.repeat_battling_has_ended: Template = {
            "image": cv2.imread(self.dirPath + '/../images/hunt/repeat_battling_has_ended.png', 0),
            "name": "repeat_battling_has_ended"
        }
        self.stage_clear: Template = {
            "image": cv2.imread(self.dirPath + '/../images/hunt/stage_clear.png', 0),
            "name": "stage_clear"
        }


from time import sleep
from epic7_bot.modules.Module import Module


class Summon(Module):
    def __init__(self):
        super(self.__class__, self).__init__()

    def get_free_summon(self):
        self.ScreenManager.ensure_not_on_sleep_mode_on_lobby()

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=245, y1=801, x2=349, y2=878, action="Click on Summon Lobby Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1232, y1=649, x2=1478, y2=685, action="Click on Covenant Summon Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=433, y1=795, x2=554, y2=851, action="Click on Free Summon Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=889, y1=636, x2=1010, y2=678, action="Click on Confirm Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=702, y1=453, x2=794, y2=508, action="Click on Middle of Screen")

        sleep(4)

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=143, y1=809, x2=251, y2=842, action="Click on Back Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=34, y1=12, x2=225, y2=67, action="Go Back to Lobby")



import logging
import subprocess
from time import sleep
from ppadb.client import Client
from epic7_bot.utils.Singleton import Singleton


class DeviceManager(metaclass=Singleton):
    def __init__(self):
        self.device = self.setup_device()

    def setup_device(self):
        self.ensure_adb_is_running()
        client = Client(host="127.0.0.1", port=5037)
        devices = client.devices()
        if len(devices) == 0:
            logging.error("No device found")
            self.ensure_device_is_connected()
            sleep(2)
            devices = client.devices()

        device = devices[0]
        device.shell("wm size 1600x900")
        return device

    def ensure_adb_is_running(self):
        logging.info("Ensure ADB is running")
        subprocess.run(["adb", "start-server"])

    def ensure_device_is_connected(self):
        logging.info("Trying to connect to a Nox emulator device")
        subprocess.run(["adb", "connect", "127.0.0.1:62001"])
        logging.info("Trying to connect to a Bluestacks emulator device")
        subprocess.run(["adb", "connect", "127.0.0.1:5555"])



from epic7_bot.commands.Command import Command
from epic7_bot.modules.Battle import Battle


class Hunt(Command):
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.Battle = Battle()

    def start(self):
        if self.currentScreen == False:
            self.Battle.start_hunt_from_lobby()
        else:
            self.Battle.start_hunt()



import logging
import time
from epic7_bot.core.DeviceManager import DeviceManager
from epic7_bot.modules.Module import Module
from epic7_bot.templates.CommonTemplates import CommonTemplates


class Arena(Module):
    def __init__(self):
        super(self.__class__, self).__init__()
        self.DeviceManager = DeviceManager()
        self.CommonTemplates = CommonTemplates()

    def battle_rotation(self):
        logging.info(f"Started battle rotation")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1065, x2=1216, y1=799, y2=852, action="Click on start battle")

        self.ScreenManager.sleep(4)

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1476, x2=1574, y1=23, y2=76, action="Click on skip")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1379, x2=1439, y1=14, y2=68, action="Click on auto battle")

        logging.info(f"Wait for skip button to appear")
        while self.ScreenManager.match_template_on_screen_area(x1=1471, x2=1581, y1=19, y2=76,
                                                               template=self.CommonTemplates.skip_button, percentage=0.55) is None:
            self.ScreenManager.sleep(1)

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1476, x2=1574, y1=23, y2=76, action="Click on skip button")

        self.ScreenManager.sleep(2)

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1378, x2=1546, y1=802, y2=853, action="Click on confirm")

    def do_battle_rotation(self, x1, y1, x2, y2, action):
        clicked = self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1, y1, x2, y2, action, percentage=50)
        if clicked:
            self.battle_rotation()

    def scroll(self):
        self.DeviceManager.device.shell(
            "input touchscreen swipe 1200 700 1200 400 200")

    def scroll_and_do_battle_rotation(self, x1, y1, x2, y2, action):
        self.scroll()
        self.do_battle_rotation(x1, y1, x2, y2, action)

    def start_arena_npc_auto_battle(self):
        self.do_battle_rotation(x1=1109, x2=1212, y1=218, y2=294,
                                action="Click on first opponent")

        self.do_battle_rotation(x1=1115, x2=1208, y1=354, y2=418,
                                action="Click on second opponent")

        self.do_battle_rotation(x1=1117, x2=1203, y1=480, y2=544,
                                action="Click on third opponent")

        self.do_battle_rotation(x1=1117, x2=1207, y1=615, y2=681,
                                action="Click on fouth opponent")

        self.do_battle_rotation(x1=1114, x2=1212, y1=740, y2=815,
                                action="Click on fifth opponent")

        self.scroll_and_do_battle_rotation(x1=1121, x2=1209, y1=274, y2=339,
                                           action="Click on sixty opponent")

        self.scroll_and_do_battle_rotation(x1=1117, x2=1208, y1=408, y2=479,
                                           action="Click on seventy opponent")

        self.scroll_and_do_battle_rotation(x1=1117, x2=1203, y1=546, y2=594,
                                           action="Click on eighth opponent")

        self.scroll_and_do_battle_rotation(x1=1117, x2=1207, y1=676, y2=735,
                                           action="Click on nineth opponent")

        self.scroll_and_do_battle_rotation(x1=1114, x2=1212, y1=804, y2=865,
                                           action="Click on tenth opponent")

    def start_arena_npc_auto_battle_from_lobby(self):
        self.ScreenManager.ensure_not_on_sleep_mode_on_lobby()

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1022, x2=1129, y1=749, y2=882, action="Click on arena icon on lobby")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=246, x2=438, y1=218, y2=283, action="Click on arena ranked")

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=1334, x2=1551, y1=236, y2=298, action="Click on NPC opponents")

        self.start_arena_npc_auto_battle()



from time import sleep
from epic7_bot.modules.Module import Module


class Guild(Module):
    def __init__(self):
        super(self.__class__, self).__init__()

    def world_boss_rotation(self):
        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1083, y1=808, x2=1182, y2=848, action="Click on Ready Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1221, y1=739, x2=1390, y2=774, action="Click on Select Team Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1249, y1=805, x2=1460, y2=838, action="Click on Team Formation Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=120, y1=794, x2=287, y2=838, action="Click on Auto Assign Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1104, y1=804, x2=1189, y2=842, action="Click on Start Button")

        sleep(10)

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1318, y1=25, x2=1407, y2=62, action="Click on Skip Button")

        sleep(10)

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=735, y1=758, x2=873, y2=798, action="Click on Open All Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=735, y1=758, x2=873, y2=798, action="Click on Confirm Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1388, y1=806, x2=1505, y2=844, action="Click on Confirm Button Again")

    def do_daily_contributions(self):
        self.ScreenManager.ensure_not_on_sleep_mode_on_lobby()

        self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
            x1=604, y1=812, x2=679, y2=881, action="Click on Guild Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=759, y1=797, x2=844, y2=833, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1064, y1=792, x2=1186, y2=848, action="Click on Daily Reward")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=759, y1=797, x2=844, y2=833, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1332, y1=249, x2=1476, y2=294, action="Click on Battlefield Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=408, y1=509, x2=614, y2=556, action="Click on World Boss Button")

        self.world_boss_rotation()

        self.world_boss_rotation()

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=34, y1=12, x2=225, y2=67, action="Go Back to Clan Lobby")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1336, y1=502, x2=1441, y2=535, action="Click on Donate Section")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1041, y1=528, x2=1155, y2=559, action="Click on Donate Button")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=720, y1=649, x2=878, y2=683, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=1046, y1=799, x2=1159, y2=839, action="Click on Donate Button Again")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=720, y1=649, x2=878, y2=683, action="Click on Tap to Close")

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=34, y1=12, x2=225, y2=67, action="Go Back to Lobby")



class Spinner(object):

    def __init__(self):
        self.frames = u'|/-\\'
        self.length = len(self.frames)
        self.position = 0

    def current(self):
        return self.frames[self.position]

    def next(self):
        current_frame = self.current()
        self.position = (self.position + 1) % self.length
        return str(current_frame)

    def reset(self):
        self.position = 0




import base64
import logging
import time
import cv2
import random

from cv2 import Mat

import numpy as np
from epic7_bot.core.DeviceManager import DeviceManager
from epic7_bot.core.MathUtils import MathUtils
from epic7_bot.templates.Template import Template
from epic7_bot.utils.Singleton import Singleton


class ScreenManager(metaclass=Singleton):
    def __init__(self):
        self.MathUtils = MathUtils()
        self.DeviceManager = DeviceManager()

    def sleep(self, waitTime):
        time.sleep(random.uniform(waitTime, waitTime + 1))

    def get_position_of_template_match(self, matchTemplate):
        position_x = np.unravel_index(
            matchTemplate.argmax(), matchTemplate.shape)[1]
        position_y = np.unravel_index(
            matchTemplate.argmax(), matchTemplate.shape)[0]
        return position_x, position_y

    def take_screenshot(self):
        png_screenshot_data = self.DeviceManager.device.shell(
            "screencap -p | busybox base64")
        png_screenshot_data = base64.b64decode(png_screenshot_data)
        nparr = np.frombuffer(png_screenshot_data, np.uint8)
        img = cv2.imdecode(nparr, 0)
        return img

    def take_screnshot_from_area(self, x1=None, x2=None, y1=None, y2=None):
        img = self.take_screenshot()
        if x1 != None and y1 != None and x2 != None and y2 != None:
            img = img[y1:y2, x1:x2]

        return img

    def match_template_on_screen(self, template: Template, percentage=0.6):
        image = self.take_screenshot()
        match = cv2.matchTemplate(
            image, template['image'], cv2.TM_CCOEFF_NORMED)
        min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(match)

        logging.debug(
            f"match_template_on_screen {template['name']}, percentage: {max_val}, {max_val > percentage}")

        if max_val > percentage:
            return match
        else:
            return None

    def match_template_on_screen_area(self, x1, y1, x2, y2, template, percentage=0.55):
        image = self.take_screnshot_from_area(x1, x2, y1, y2)
        result = cv2.matchTemplate(
            image, template['image'], cv2.TM_CCOEFF_NORMED)
        min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
        logging.debug(
            f"match_template_on_screen_area {template['name']}, percentage: {max_val}, {max_val > percentage}")
        if max_val > percentage:
            return max_val
        else:
            return None

    def match_template_on_screen_and_click_it(self, template, waitTime=0):
        self.sleep(random.uniform(waitTime, waitTime + 0.5))
        match = self.match_template_on_screen(template)
        if match is not None:
            position_x, position_y = self.get_position_of_template_match(match)
            x, y = self.MathUtils.randomPoint(position_x, position_y)
            self.DeviceManager.device.shell("input tap " +
                                            str(x) + " " + str(y))

    def check_if_images_changed(self, img1, img2, percentage=70, action=None):
        if img1 is None or img2 is None:
            return False
        res = cv2.absdiff(img1, img2)
        res = res.astype(np.uint8)
        result = (np.count_nonzero(res) * 100) / res.size
        if action is not None:
            logging.debug(f"{action}, check_if_images_changed: {result}")
        return result >= percentage

    def click_position(self, position_x, position_y, waitTime, message=None):
        self.sleep(waitTime)
        x, y = self.MathUtils.randomPoint(position_x, position_y)
        self.DeviceManager.device.shell("input tap " + str(x) + " " + str(y))

    def random_click_on_area(self, x1, y1, x2, y2):
        position_x, position_y = self.MathUtils.random_point_in_area(
            x1, y1, x2, y2)
        self.click_position(position_x, position_y, waitTime=0)
        # position_x, position_y = self.MathUtils.midpoint(x1, y1, x2, y2)
        # self.click_position(position_x, position_y, waitTime=0)

    def random_click_on_area_get_before_and_after_images_from_screen(self, x1, y1, x2, y2):
        beforeImage = self.take_screenshot()
        self.random_click_on_area(x1, y1, x2, y2)
        self.sleep(3)
        afterImage = self.take_screenshot()
        return (beforeImage, afterImage)

    def random_click_on_area_and_check_change_on_screen_retry(self, x1, y1, x2, y2, action=None, percentage=70):
        self.sleep(1)
        if action is not None:
            logging.info(f"{action}")

        beforeImage, afterImage = None, None
        count = 0
        while self.check_if_images_changed(beforeImage, afterImage, percentage=percentage, action=action) is False and count < 2:
            beforeImage, afterImage = self.random_click_on_area_get_before_and_after_images_from_screen(
                x1, y1, x2, y2)
            count += 1
        if count < 2 == False:
            logging.error(f"{action}")

        return count < 2

    def random_click_on_area_get_before_and_after_images_from_area(self, x1, y1, x2, y2):
        beforeImage = self.take_screnshot_from_area(x1, x2, y1, y2)
        self.random_click_on_area(x1, y1, x2, y2)
        self.sleep(3)
        afterImage = self.take_screnshot_from_area(x1, x2, y1, y2)
        return (beforeImage, afterImage)

    def random_click_on_area_and_check_change_on_area_retry(self, x1, y1, x2, y2, action=None, percentage=70):
        self.sleep(1)
        if action is not None:
            logging.info(f"{action}")

        beforeImage, afterImage = None, None
        count = 0
        while self.check_if_images_changed(beforeImage, afterImage, percentage, action) is False and count < 2:
            beforeImage, afterImage = self.random_click_on_area_get_before_and_after_images_from_area(
                x1, y1, x2, y2)
            count += 1
        return count < 2

    def ensure_not_on_sleep_mode_on_lobby(self):
        logging.info(
            "Double Click on Screen to Ensure not on Sleep Mode on Lobby")
        self.random_click_on_area(
            x1=894, y1=848, x2=935, y2=879)
        self.sleep(1)
        self.random_click_on_area(
            x1=894, y1=848, x2=935, y2=879)



"""

Usage:
    epic7 <command> [options]

All commands should be run when the game screen is in the lobby!

The most commonly used commands are:
    shop            Start secret shop auto buy
    arena           Start arena npc auto battle
    hunt            Start hunt auto battle
    daily           Start daily actions

Options:
    -h --help       Show this help message and exit
    -v --version    Show version and exit
    -c --current    Run command on current screen
"""

import sys
from time import sleep
from docopt import docopt
from sys import exit
from epic7_bot.processes.CheckConnection import CheckConnection
from epic7_bot.processes.CommandRunner import CommandRunner
from epic7_bot.processes.CommandRunner import CommandRunner
from epic7_bot.processes.DispatchMissionChecker import DispatchMissionChecker


def main():
    try:
        args = docopt(__doc__, version="1.0.0", options_first=False)

        commandRunner = CommandRunner(args)
        dispatchMissionChecker = DispatchMissionChecker(commandRunner.pid)
        checkConnection = CheckConnection(commandRunner.pid)

        if args['<command>'] in [None]:
            sys.argv.append('-h')
            exit(main())
        elif args['<command>'] not in commandRunner.commands.keys():
            exit("%r is not a valid command." %
                 args['<command>'])
        else:
            commandRunner.start()
            sleep(3)
            checkConnection.start()
            dispatchMissionChecker.start()
            while True:
                if commandRunner.is_alive() is False:
                    print("\n\nJob Finished, bot closing")

                    exit()
                sleep(1)
    except KeyboardInterrupt:
        print("\n\nCtrol-C pressed, bot closing")

        while True:
            if commandRunner.is_alive():
                commandRunner.terminate()
            if checkConnection.is_alive():
                checkConnection.terminate()
            if commandRunner.is_alive() is not True and commandRunner.is_alive() is not True:
                exit()
            sleep(0.5)



import math
from random import random as rand
import random


from epic7_bot.utils.Singleton import Singleton


class MathUtils(metaclass=Singleton):
    def randomPoint(self, aroundX, aroundY, scale=1, density=2):
        angle = rand()*2*math.pi

        x = rand()
        if x == 0:
            x = 0.0000001

        distance = scale * (pow(x, -1.0/density) - 1)
        return (aroundX + distance * math.sin(angle),
                aroundY + distance * math.cos(angle))

    def midpoint(self, x1, y1, x2, y2):
        return ((x1 + x2)/2, (y1 + y2)/2)

    def random_point_in_area(self, x1, y1, x2, y2):
        x = random.uniform(x1, x2)
        y = random.uniform(y1, y2)
        return x, y




import multiprocessing
from epic7_bot.commands.Command import Command
from epic7_bot.core.Logger import get_log_level, init_logger
from dotenv import load_dotenv
from epic7_bot.modules.Arena import Arena
from epic7_bot.commands.Daily import Daily
from epic7_bot.commands.Hunt import Hunt
from epic7_bot.commands.Arena import Arena
from epic7_bot.commands.Shop import Shop


class CommandRunner(multiprocessing.Process):
    def __init__(self, args):
        multiprocessing.Process.__init__(self, daemon=True)
        self.args = args
        self.commands: dict[str, type[Command]] = {
            "daily": Daily,
            "hunt": Hunt,
            "arena": Arena,
            "shop": Shop,
        }

    def run(self):
        try:
            load_dotenv()
            init_logger()

            command = self.commands[self.args['<command>']]
            command(currentScreen=self.args['--current']).start()
        except BaseException as e:
            if get_log_level() == "DEBUG":
                raise e
            else:
                pass



import logging
import multiprocessing
import sys
import time
from epic7_bot.core.Logger import get_log_level
from epic7_bot.core.ScreenManager import ScreenManager
from epic7_bot.templates.CommonTemplates import CommonTemplates
import psutil


class CheckConnection(multiprocessing.Process):
    def __init__(self, main_process_pid):
        multiprocessing.Process.__init__(self, daemon=True)
        self.CommonTemplates = CommonTemplates()
        self.ScreenManager = ScreenManager()
        self.main_process_pid = main_process_pid

    def match_there_was_a_connection_error_template(self):
        return self.ScreenManager.match_template_on_screen_area(
            x1=475, y1=372, x2=1122, y2=406, template=self.CommonTemplates.there_was_a_connection_error, percentage=0.7)

    def match_connecting_problem_template(self):
        return self.ScreenManager.match_template_on_screen_area(
            x1=694, x2=908, y1=421, y2=451, template=self.CommonTemplates.connecting_problem, percentage=0.7)

    def run(self):
        try:
            main_process = psutil.Process(self.main_process_pid)
            seconds = 21600  # 6 hours

            # problems with - RecursionError: maximum recursion depth exceeded in comparison
            while seconds != 0:
                seconds -= 1
                time.sleep(1)
                there_was_a_connection_error = self.match_there_was_a_connection_error_template()
                connecting_problem = self.match_connecting_problem_template()
                if connecting_problem is not None or there_was_a_connection_error is not None:
                    logging.error(f"Found Connection Problem")
                    main_process.suspend()
                    while True:
                        time.sleep(1)
                        if there_was_a_connection_error is not None:
                            there_was_a_connection_error = self.match_there_was_a_connection_error_template()
                            self.ScreenManager.random_click_on_area_and_check_change_on_area_retry(
                                x1=475, y1=372, x2=1122, y2=406, action="Click on there_was_a_connection_error")
                            if there_was_a_connection_error is None:
                                main_process.resume()
                                break
                        if connecting_problem is not None:
                            connecting_problem = self.match_connecting_problem_template()
                            if connecting_problem is None:
                                main_process.resume()
                                break
        except BaseException as e:
            if get_log_level() == "DEBUG":
                raise e
            else:
                pass



from epic7_bot.commands.Command import Command
from epic7_bot.modules.Guild import Guild
from epic7_bot.modules.PetHouse import PetHouse
from epic7_bot.modules.Sanctuary import Sanctuary
from epic7_bot.modules.Summon import Summon


class Daily(Command):
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.Summon = Summon()
        self.PetHouse = PetHouse()
        self.Guild = Guild()
        self.Sanctuary = Sanctuary()

    def start(self):
        # TODO: Sanctuary needs template matching
        # self.Sanctuary.do_sanctuary_routine()
        self.Summon.get_free_summon()
        self.PetHouse.get_free_pet_summon()
        self.Guild.do_daily_contributions()



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="epic7_bot",
    version="1.0.1",
    description="Epic7 Bot used via cli",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="brunocordioli072",
    packages=["epic7_bot"].extend(find_packages("epic7_bot")),
    package_data={'': ['images/**/*.png']},
    include_package_data=True,
    install_requires=["numpy ==1.23.2",
                      "opencv_python ==4.6.0.66",
                      "pure_python_adb ==0.3.0.dev0",
                      "docopt ==0.6.2",
                      "python-dotenv ==0.21.0",
                      "psutil ==5.9.3"],
    entry_points={
        "console_scripts": [
            "epic7=epic7_bot.cli:main",
        ],
    },
    setup_requires=[
        "setuptools >= 49.2.1",
        "wheel"
    ],
)



import logging
import math
from epic7_bot.core.DeviceManager import DeviceManager
from epic7_bot.core.MathUtils import MathUtils
from epic7_bot.modules.Module import Module
from epic7_bot.templates.SecretShopTemplates import SecretShopTemplates


class SecretShop(Module):
    def __init__(self):
        super(self.__class__, self).__init__()
        self.DeviceManager = DeviceManager()
        self.SecretShopTemplates = SecretShopTemplates()
        self.MathUtils = MathUtils()

        self.mystic_count = 0
        self.refreshes_count = 0
        self.covenant_count = 0
        self.bought = False

    def check_bookmarks(self):
        logging.info("Check Mystic and Covenant Bookmarks")
        mystic_pos = self.ScreenManager.match_template_on_screen(
            self.SecretShopTemplates.mystic)
        coven_pos = self.ScreenManager.match_template_on_screen(
            self.SecretShopTemplates.covenant)

        if mystic_pos is not None:
            logging.info("Found Mystic Bookmark")
            x, y = self.ScreenManager.get_position_of_template_match(
                mystic_pos)

            width, height = self.SecretShopTemplates.mystic["image"].shape[::-1]

            x1 = x + math.floor(width / 2) + 580
            y1 = y + math.floor(height / 2) + 10
            x2 = x + math.floor(width / 2) + 800
            y2 = y + math.floor(height / 2) + 55

            self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
                x1=x1, y1=y1, x2=x2, y2=y2, action="Click on Buy")

            self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
                x1=761, y1=605, x2=1059, y2=660, action="Click on Confirm Buy")

            self.bought = True
            self.mystic_count += 1

        if coven_pos is not None:
            logging.info("Found Covenant Bookmark")
            x, y = self.ScreenManager.get_position_of_template_match(coven_pos)

            width, height = self.SecretShopTemplates.covenant["image"].shape[::-1]

            x1 = x + math.floor(width / 2) + 580
            y1 = y + math.floor(height / 2) + 10
            x2 = x + math.floor(width / 2) + 800
            y2 = y + math.floor(height / 2) + 55

            self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
                x1=x1, y1=y1, x2=x2, y2=y2, action="Click on Buy")

            self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
                x1=761, y1=605, x2=1059, y2=660, action="Click on Confirm Buy")

            self.bought = True
            self.covenant_count += 1

    def scroll(self):
        x1, y1 = self.MathUtils.random_point_in_area(
            x1=750, y1=781, x2=1282, y2=845)
        x2, y2 = self.MathUtils.random_point_in_area(
            x1=701, y1=98, x2=1288, y2=174)

        self.DeviceManager.device.shell(
            f"input touchscreen swipe {x1} {y1} {x2} {y2} 400")

    def check_store(self):
        self.check_bookmarks()

        if self.bought is False:
            self.scroll()
            self.ScreenManager.sleep(1)
            self.check_bookmarks()
        self.bought = False

    def refresh(self):
        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=287, y1=808, x2=387, y2=838, action="Click on Refresh Button")
        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=878, y1=537, x2=986, y2=568, action="Click on Confirm Button")

        self.refreshes_count += 1

    def show_stats(self):
        comment = f"\nTotal Covenant: {str(self.covenant_count)}{' '*30}" + \
            f"\nTotal Mystic: {str(self.mystic_count)}{' '*30}" + \
            f"\nTotal Refreshes: {str(self.refreshes_count)}{' '*30}\n\n\n"
        previus_line = "\033[F"
        if self.refreshes_count == 1:
            print(f"\r{previus_line*2}{comment}")
        else:
            print(f"\r{previus_line*7}{comment}")
        pass

    def start_auto_buy_secret_shop(self):
        while True:
            self.check_store()
            self.refresh()
            self.show_stats()

    def start_auto_buy_secret_shop_from_lobby(self):
        self.ScreenManager.ensure_not_on_sleep_mode_on_lobby()

        self.ScreenManager.random_click_on_area_and_check_change_on_screen_retry(
            x1=683, y1=204, x2=728, y2=246, action="Click on Bartender")
        self.start_auto_buy_secret_shop()



import logging
import os
import io
import logging
import sys
import threading
import datetime

from epic7_bot.utils.Spinner import Spinner


def get_log_level():
    LOG_LEVEL = os.getenv("LOG_LEVEL") if os.getenv(
        "LOG_LEVEL") is not None else "INFO"
    return LOG_LEVEL


def init_logger():
    level = logging.getLevelName(get_log_level())
    logging.basicConfig(level=level, handlers=[LoggerHandler(level=level)])


class LoggerHandler(logging.Handler):

    def __init__(self,
                 stream=None,
                 spin_interval=0.1,
                 format=u'{message} {spinner}',
                 level=logging.NOTSET):
        super(LoggerHandler, self).__init__(level)
        self._stream = stream
        self._message_format = format
        self._spinner = Spinner()
        self._spin_interval = spin_interval
        self._current_record_changed = threading.Condition()
        self._thread = threading.Thread(target=self._display, daemon=True)
        self._current_record = None
        sys.stdout.write('\n')
        sys.stdout.flush()

    def get_stream(self):
        stream = sys.stdout if self._stream is None else self._stream
        if callable(getattr(stream, 'isatty')):
            atty = stream.isatty()
        else:
            atty = False
        try:
            stream.write('')
        except TypeError:
            stream = io.TextIOWrapper(stream)
        return stream, atty

    def emit(self, record):
        if not self.filter(record):
            return
        stream, atty = self.get_stream()
        if not atty:
            stream.write(record.getMessage())
            return
        with self._current_record_changed:
            self._current_record = record
            self._current_record_changed.notify()
        if not self._thread.is_alive():
            self._thread.start()

    def _display(self):
        stream, _ = self.get_stream()
        format_message = self._message_format.format
        if self.level == 10:  # debug level
            stream.write(
                f"{datetime.datetime.now()} :: {self._current_record.getMessage()} \n")
            stream.flush()
        else:
            format_line = u'\r\033[F{0:{1}}\n\x1B[3mUse Ctrol-C to stop program.\x1B[0m'.format
            previous_line_length = 0
            while True:
                record = self._current_record

                if record is None:
                    break
                s = format_message(spinner=self._spinner.next(),
                                   record=record,
                                   message=record.getMessage())
                stream.write(format_line(s, previous_line_length or 1))
                previous_line_length = len(s)
                stream.flush()
                with self._current_record_changed:
                    self._current_record_changed.wait(self._spin_interval)
            s = self._current_record.getMessage()
            stream.write(u'\r{0:{1}}\n'.format(s, previous_line_length or 1))
            stream.flush()
        self._thread = threading.Thread(target=self._display, daemon=True)
